declare namespace _default {
    export const LEFT: number;
    export const UP: number;
    export const RIGHT: number;
    export const DOWN: number;
}
export default _default;
//# sourceMappingURL=KeyCode.d.ts.map