declare namespace _default {
    export const POINTERMOVE: string;
    export const POINTERDOWN: string;
    export const POINTERUP: string;
    export const POINTEROVER: string;
    export const POINTEROUT: string;
    export const POINTERENTER: string;
    export const POINTERLEAVE: string;
    export const POINTERCANCEL: string;
}
export default _default;
//# sourceMappingURL=EventType.d.ts.map