declare namespace _default {
    export const IDLE: number;
    export const LOADING: number;
    export const LOADED: number;
    export const ERROR: number;
    export const EMPTY: number;
}
export default _default;
//# sourceMappingURL=ImageState.d.ts.map