declare namespace _default {
    export const CARMENTA_SERVER: string;
    export const GEOSERVER: string;
    export const MAPSERVER: string;
    export const QGIS: string;
}
export default _default;
//# sourceMappingURL=WMSServerType.d.ts.map