<?php

// Require
require_once '/usr/share/php/Composer/CaBundle/autoload.php';
require_once '/usr/share/php/FastRoute/autoload.php';
require_once '/usr/share/php/PhpMyAdmin/MoTranslator/autoload.php';
require_once '/usr/share/php/PhpMyAdmin/ShapeFile/autoload.php';
require_once '/usr/share/php/PhpMyAdmin/SqlParser/autoload.php';
require_once '/usr/share/php/PhpMyAdmin/Twig/Extensions/autoload.php';
require_once '/usr/share/php/Psr/Http/Message/autoload.php';
require_once '/usr/share/php/Psr/Http/Message/factory-autoload.php';
require_once '/usr/share/php/ReCaptcha/autoload.php';
require_once '/usr/share/php/Slim/Psr7/autoload.php';
require_once '/usr/share/php/Symfony/Component/Config/autoload.php';
require_once '/usr/share/php/Symfony/Component/DependencyInjection/autoload.php';
require_once '/usr/share/php/Symfony/Component/ExpressionLanguage/autoload.php';
require_once '/usr/share/php/Symfony/Polyfill/Php80/autoload.php';
require_once '/usr/share/php/Twig/autoload.php';
require_once '/usr/share/php/Webmozart/Assert/autoload.php';
require_once '/usr/share/php/Williamdes/MariaDBMySQLKBS/autoload.php';

// Suggest
if (stream_resolve_include_path('/usr/share/php/Bacon/BaconQrCode/autoload.php')) { include_once '/usr/share/php/Bacon/BaconQrCode/autoload.php'; }
if (stream_resolve_include_path('/usr/share/php/CodeLts/U2F-php-server/autoload.php')) { include_once '/usr/share/php/CodeLts/U2F-php-server/autoload.php'; }
if (stream_resolve_include_path('/usr/share/php/Pragmarx/Google2faQrcode/autoload.php')) { include_once '/usr/share/php/Pragmarx/Google2faQrcode/autoload.php'; }// TODO -> package missing
if (stream_resolve_include_path('/usr/share/php/tcpdf/autoload.php')) { include_once '/usr/share/php/tcpdf/autoload.php'; }

// @codingStandardsIgnoreFile
// @codeCoverageIgnoreStart
// this is an autogenerated file - do not edit
spl_autoload_register(
    function($class) {
        static $classes = null;
        if ($classes === null) {
            $classes = array(
                ___CLASSLIST___
            );
        }
        $cn = strtolower($class);
        if (isset($classes[$cn])) {
            require ___BASEDIR___$classes[$cn];
        }
    },
    ___EXCEPTION___,
    ___PREPEND___
);
// @codeCoverageIgnoreEnd

// Files
require_once '/usr/share/php/PhpMyAdmin/MoTranslator/functions.php';
