declare namespace _default {
    export const CHANGE: string;
    export const ERROR: string;
    export const BLUR: string;
    export const CLEAR: string;
    export const CONTEXTMENU: string;
    export const CLICK: string;
    export const DBLCLICK: string;
    export const DRAGENTER: string;
    export const DRAGOVER: string;
    export const DROP: string;
    export const FOCUS: string;
    export const KEYDOWN: string;
    export const KEYPRESS: string;
    export const LOAD: string;
    export const RESIZE: string;
    export const TOUCHMOVE: string;
    export const WHEEL: string;
}
export default _default;
//# sourceMappingURL=EventType.d.ts.map