declare namespace _default {
    export const UNKNOWN: number;
    export const INTERSECTING: number;
    export const ABOVE: number;
    export const RIGHT: number;
    export const BELOW: number;
    export const LEFT: number;
}
export default _default;
//# sourceMappingURL=Relationship.d.ts.map