declare namespace _default {
    export const POINT: string;
    export const LINE_STRING: string;
    export const LINEAR_RING: string;
    export const POLYGON: string;
    export const MULTI_POINT: string;
    export const MULTI_LINE_STRING: string;
    export const MULTI_POLYGON: string;
    export const GEOMETRY_COLLECTION: string;
    export const CIRCLE: string;
}
export default _default;
//# sourceMappingURL=GeometryType.d.ts.map