declare namespace _default {
    export const IMAGE: string;
    export const HYBRID: string;
    export const VECTOR: string;
}
export default _default;
//# sourceMappingURL=VectorTileRenderType.d.ts.map