declare namespace _default {
    export const TILELOADSTART: string;
    export const TILELOADEND: string;
    export const TILELOADERROR: string;
}
export default _default;
export type TileSourceEventTypes = "tileloadstart" | "tileloadend" | "tileloaderror";
//# sourceMappingURL=TileEventType.d.ts.map